@@ -8,12 +8,6 @@
 #include <string.h>
 #include <unistd.h>
 
-#define LIBSOUP_USE_UNSTABLE_REQUEST_API
-#include <libsoup/soup-multipart-input-stream.h>
-#include <libsoup/soup-request-http.h>
-#include <libsoup/soup-requester.h>
-#include <libsoup/soup.h>
-
 #include "test-utils.h"
 
 #define READ_BUFFER_SIZE 8192
@@ -26,7 +20,7 @@ typedef enum {
 } MultipartMode;
 
 char *buffer;
-SoupRequester *requester;
+SoupSession *session;
 char *base_uri_string;
 SoupURI *base_uri;
 SoupMultipartInputStream *multipart;
@@ -136,16 +130,19 @@ read_cb (GObject *source, GAsyncResult *asyncResult, gpointer data)
 	GInputStream *stream = G_INPUT_STREAM (source);
 	GError *error = NULL;
 	gssize bytes_read = g_input_stream_read_finish (stream, asyncResult, &error);
+
 	if (error) {
 		debug_printf (1, "  failed read: %s\n", error->message);
 		errors++;
 
+		g_object_unref (stream);
 		g_main_loop_quit (loop);
 		return;
 	}
 
 	if (!bytes_read) {
 		g_input_stream_close (stream, NULL, &error);
+		g_object_unref (stream);
 
 		if (error) {
 			debug_printf (1, "  failed close: %s\n", error->message);
@@ -252,6 +249,7 @@ multipart_read_cb (GObject *source, GAsyncResult *asyncResult, gpointer data)
 
 		g_input_stream_close_async (in, G_PRIORITY_DEFAULT, NULL,
 					    multipart_close_part_cb, NULL);
+		g_object_unref (in);
 
 		g_main_loop_quit (loop);
 		return;
@@ -265,6 +263,7 @@ multipart_read_cb (GObject *source, GAsyncResult *asyncResult, gpointer data)
 
 		g_input_stream_close_async (in, G_PRIORITY_DEFAULT, NULL,
 					    multipart_close_part_cb, NULL);
+		g_object_unref (in);
 
 		soup_multipart_input_stream_next_part_async (multipart, G_PRIORITY_DEFAULT, NULL,
 							     multipart_next_part_cb, data);
@@ -368,8 +367,10 @@ multipart_next_part_cb (GObject *source, GAsyncResult *res, gpointer data)
 
 	if (error) {
 		debug_printf (1, "  failed next part: %s\n", error->message);
+		g_clear_error (&error);
 		errors++;
 
+		g_object_unref (multipart);
 		g_main_loop_quit (loop);
 		return;
 	}
@@ -380,6 +381,7 @@ multipart_next_part_cb (GObject *source, GAsyncResult *res, gpointer data)
 			errors++;
 		}
 
+		g_object_unref (multipart);
 		g_main_loop_quit (loop);
 		return;
 	}
@@ -406,6 +408,8 @@ multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 	in = soup_request_send_finish (request, res, &error);
 	message = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
 	multipart = soup_multipart_input_stream_new (message, in);
+	g_object_unref (message);
+	g_object_unref (in);
 
 	if (error) {
 		debug_printf (1, "  failed send: %s\n", error->message);
@@ -436,6 +440,8 @@ sync_multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 	in = soup_request_send_finish (request, res, &error);
 	message = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
 	multipart = soup_multipart_input_stream_new (message, in);
+	g_object_unref (message);
+	g_object_unref (in);
 
 	if (error) {
 		debug_printf (1, "  failed send: %s\n", error->message);
@@ -466,12 +472,14 @@ sync_multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 			debug_printf (1, "  failed sync read: %s\n", error->message);
 			errors++;
 			g_clear_error (&error);
+			g_object_unref (in);
 			break;
 		}
 
 		check_read (bytes_read, passes);
 
 		passes++;
+		g_object_unref (in);
 	}
 
 	if (passes != 4) {
@@ -480,7 +488,7 @@ sync_multipart_handling_cb (GObject *source, GAsyncResult *res, gpointer data)
 	}
 
 	g_main_loop_quit (loop);
-
+	g_object_unref (multipart);
 }
 
 static const char*
@@ -500,7 +508,7 @@ static void
 test_multipart (int headers_expected, int sniffed_expected, MultipartMode multipart_mode)
 {
 	GError* error = NULL;
-	SoupRequest* request = soup_requester_request (requester, base_uri_string, &error);
+	SoupRequest* request = soup_session_request (session, base_uri_string, &error);
 
 	SoupMessage *msg = soup_request_http_get_message (SOUP_REQUEST_HTTP (request));
 	GMainLoop *loop = g_main_loop_new (NULL, TRUE);
@@ -525,8 +533,6 @@ test_multipart (int headers_expected, int sniffed_expected, MultipartMode multip
 	g_signal_connect (msg, "content-sniffed",
 			  G_CALLBACK (content_sniffed), &sniffed_count);
 
-	g_object_ref (msg);
-
 	if (multipart_mode == ASYNC_MULTIPART)
 		soup_request_send_async (request, NULL, multipart_handling_cb, loop);
 	else if (multipart_mode == ASYNC_MULTIPART_SMALL_READS) {
@@ -577,7 +583,6 @@ int
 main (int argc, char **argv)
 {
 	SoupServer *server;
-	SoupSession *session;
 
 	test_init (argc, argv, NULL);
 
@@ -597,11 +602,8 @@ main (int argc, char **argv)
 					 "max-conns", 20,
 					 "max-conns-per-host", 20,
 					 NULL);
-	soup_session_add_feature_by_type (session, SOUP_TYPE_REQUESTER);
 	soup_session_add_feature_by_type (session, SOUP_TYPE_CONTENT_SNIFFER);
 
-	requester = SOUP_REQUESTER (soup_session_get_feature (session, SOUP_TYPE_REQUESTER));
-
 	test_multipart (1, 1, NO_MULTIPART);
 	test_multipart (1, 1, SYNC_MULTIPART);
 	test_multipart (1, 1, ASYNC_MULTIPART);
@@ -609,6 +611,7 @@ main (int argc, char **argv)
 
 	soup_uri_free (base_uri);
 	g_free (base_uri_string);
+	g_free (buffer);
 
 	soup_test_session_abort_unref (session);
 	soup_test_server_quit_unref (server);
