@@ -19,12 +19,16 @@ typedef enum {
 	NTLM_AUTHENTICATED_BOB
 } NTLMServerState;
 
+static const char *state_name[] = {
+	"unauth", "recv", "sent", "alice", "bob"
+};
+
 #define NTLM_REQUEST_START "TlRMTVNTUAABAAAA"
 #define NTLM_RESPONSE_START "TlRMTVNTUAADAAAA"
 
 #define NTLM_CHALLENGE "TlRMTVNTUAACAAAADAAMADAAAAABAoEAASNFZ4mrze8AAAAAAAAAAGIAYgA8AAAARABPAE0AQQBJAE4AAgAMAEQATwBNAEEASQBOAAEADABTAEUAUgBWAEUAUgAEABQAZABvAG0AYQBpAG4ALgBjAG8AbQADACIAcwBlAHIAdgBlAHIALgBkAG8AbQBhAGkAbgAuAGMAbwBtAAAAAAA="
 
-#define NTLM_RESPONSE_USER(response) ((response)[86] == 'E' ? NTLM_AUTHENTICATED_ALICE : NTLM_AUTHENTICATED_BOB)
+#define NTLM_RESPONSE_USER(response) ((response)[86] == 'E' ? NTLM_AUTHENTICATED_ALICE : ((response)[86] == 'I' ? NTLM_AUTHENTICATED_BOB : NTLM_UNAUTHENTICATED))
 
 static void
 clear_state (gpointer connections, GObject *ex_connection)
@@ -134,16 +138,22 @@ server_callback (SoupServer *server, SoupMessage *msg,
 		}
 	}
 
+	debug_printf (2, " (S:%s)", state_name[state]);
 	g_hash_table_insert (connections, socket, GINT_TO_POINTER (state));
 	g_object_weak_ref (G_OBJECT (socket), clear_state, connections);
 }
 
+static gboolean authenticated_ntlm = FALSE;
+
 static void
 authenticate (SoupSession *session, SoupMessage *msg,
 	      SoupAuth *auth, gboolean retrying, gpointer user)
 {
-	if (!retrying)
+	if (!retrying) {
 		soup_auth_authenticate (auth, user, "password");
+		if (g_str_equal (soup_auth_get_scheme_name (auth), "NTLM"))
+			authenticated_ntlm = TRUE;
+	}
 }
 
 typedef struct {
@@ -313,7 +323,8 @@ do_message (SoupSession *session, SoupURI *base_uri, const char *path,
 }
 
 static void
-do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm, const char *user)
+do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm,
+	       const char *user, gboolean use_builtin_ntlm)
 {
 	SoupSession *session;
 	gboolean alice = !g_strcmp0 (user, "alice");
@@ -322,13 +333,23 @@ do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm, const char *user)
 	gboolean bob_via_ntlm = use_ntlm && bob;
 	gboolean alice_via_basic = !use_ntlm && alice;
 
-	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
-	if (use_ntlm)
-		soup_session_add_feature_by_type (session, SOUP_TYPE_AUTH_NTLM);
+	session = soup_test_session_new (SOUP_TYPE_SESSION, NULL);
 
 	if (user) {
 		g_signal_connect (session, "authenticate",
 				  G_CALLBACK (authenticate), (char *)user);
+		if (use_ntlm && !use_builtin_ntlm)
+			g_setenv ("NTLMUSER", user, TRUE);
+	}
+	if (use_ntlm) {
+		SoupAuthManager *auth_manager;
+		SoupAuth *ntlm;
+
+		soup_session_add_feature_by_type (session, SOUP_TYPE_AUTH_NTLM);
+		auth_manager = SOUP_AUTH_MANAGER (soup_session_get_feature (session, SOUP_TYPE_AUTH_MANAGER));
+		ntlm = g_object_new (SOUP_TYPE_AUTH_NTLM, NULL);
+		soup_auth_manager_use_auth (auth_manager, base_uri, ntlm);
+		g_object_unref (ntlm);
 	}
 
 	/* 1. Server doesn't request auth, so both get_ntlm_prompt and
@@ -336,11 +357,19 @@ do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm, const char *user)
 	 * if we're using NTLM we'll try that even without the server
 	 * asking.
 	 */
+	authenticated_ntlm = FALSE;
 	do_message (session, base_uri, "/noauth",
 		    FALSE, use_ntlm,
 		    FALSE, FALSE,
 		    SOUP_STATUS_OK);
 
+	if (authenticated_ntlm != (use_ntlm && use_builtin_ntlm)) {
+		debug_printf (1, "  ERROR: %s built-in NTLM support, but authenticate signal %s emitted\n",
+			      use_builtin_ntlm ? "Using" : "Not using",
+			      authenticated_ntlm ? "was" : "wasn't");
+		errors++;
+	}
+
 	/* 2. Server requires auth as Alice, so it will request that
 	 * if we didn't already authenticate the connection to her in
 	 * the previous step. If we authenticated as Bob in the
@@ -422,41 +451,59 @@ do_ntlm_round (SoupURI *base_uri, gboolean use_ntlm, const char *user)
 }
 
 static void
-do_ntlm_tests (SoupURI *base_uri)
+do_ntlm_tests (SoupURI *base_uri, gboolean use_builtin_ntlm)
 {
 	debug_printf (1, "Round 1: Non-NTLM Connection, no auth\n");
-	do_ntlm_round (base_uri, FALSE, NULL);
+	do_ntlm_round (base_uri, FALSE, NULL, use_builtin_ntlm);
 	debug_printf (1, "Round 2: NTLM Connection, user=alice\n");
-	do_ntlm_round (base_uri, TRUE, "alice");
+	do_ntlm_round (base_uri, TRUE, "alice", use_builtin_ntlm);
 	debug_printf (1, "Round 3: NTLM Connection, user=bob\n");
-	do_ntlm_round (base_uri, TRUE, "bob");
+	do_ntlm_round (base_uri, TRUE, "bob", use_builtin_ntlm);
 	debug_printf (1, "Round 4: Non-NTLM Connection, user=alice\n");
-	do_ntlm_round (base_uri, FALSE, "alice");
+	do_ntlm_round (base_uri, FALSE, "alice", use_builtin_ntlm);
 }
 
 int
 main (int argc, char **argv)
 {
-	GMainLoop *loop;
 	SoupServer *server;
 	GHashTable *connections;
 	SoupURI *uri;
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (FALSE);
+	server = soup_test_server_new (TRUE);
 	connections = g_hash_table_new (NULL, NULL);
 	soup_server_add_handler (server, NULL,
 				 server_callback, connections, NULL);
 
-	loop = g_main_loop_new (NULL, TRUE);
-
 	uri = soup_uri_new ("http://127.0.0.1/");
 	soup_uri_set_port (uri, soup_server_get_port (server));
-	do_ntlm_tests (uri);
-	soup_uri_free (uri);
 
-	g_main_loop_unref (loop);
+	/* Built-in NTLM auth support. (We set SOUP_NTLM_AUTH_DEBUG to
+	 * an empty string to ensure that the built-in support is
+	 * being used, even if /usr/bin/ntlm_auth is available.)
+	 */
+	g_setenv ("SOUP_NTLM_AUTH_DEBUG", "", TRUE);
+	debug_printf (1, "Built-in NTLM support\n");
+	do_ntlm_tests (uri, TRUE);
+
+	/* Samba winbind /usr/bin/ntlm_auth helper support (via a
+	 * helper program that emulate's its interface).
+	 */
+	g_setenv ("SOUP_NTLM_AUTH_DEBUG", BUILDDIR "/ntlm-test-helper", TRUE);
+	debug_printf (1, "\nExternal helper support\n");
+	do_ntlm_tests (uri, FALSE);
+
+	/* Support for when ntlm_auth is installed, but the user has
+	 * no cached credentials (and thus we have to fall back to
+	 * libsoup's built-in NTLM support).
+	 */
+	g_setenv ("SOUP_NTLM_AUTH_DEBUG_NOCREDS", "1", TRUE);
+	debug_printf (1, "\nExternal -> fallback support\n");
+	do_ntlm_tests (uri, TRUE);
+
+	soup_uri_free (uri);
 
 	soup_test_server_quit_unref (server);
 	test_cleanup ();
