@@ -9,9 +9,6 @@
 
 #include "libsoup/soup.h"
 #include "libsoup/soup-requester.h"
-#include "libsoup/soup-request-data.h"
-#include "libsoup/soup-request-file.h"
-#include "libsoup/soup-request-http.h"
 
 void test_init    (int argc, char **argv, GOptionEntry *entries);
 void test_cleanup (void);
